{------------------------------------------------------------------------------}
{                                                                              }
{  TMRUFiles v1.14                                                             }
{  by Kambiz R. Khojasteh                                                      }
{                                                                              }
{  kambiz@delphiarea.com                                                       }
{  http://www.delphiarea.com                                                   }
{                                                                              }
{------------------------------------------------------------------------------}
{                                                                              }
{  Thanks to:                                                                  }
{    Wolfgang Ehrhardt for a couple of improvements                            }
{      Wolfgang.Ehrhardt@munich.netsurf.de                                     }
{      http://home.netsurf.de/wolfgang.ehrhardt                                }
{      *we 08-03: changes by W.Ehrhardt 10 Aug 2003                            }
{                                                                              }
{------------------------------------------------------------------------------}

unit MRUfiles;

interface

uses
  {$IFDEF WIN32} Windows {$ELSE} WinTypes, WinProcs {$ENDIF}, Classes, Menus;

type

  TMRUClickEvent = procedure(Sender: TObject; const FileName: String) of object;
  TMRUMenuItemEvent = procedure(Sender: TObject; Index: Integer;
    MenuItem: TMenuItem) of Object;

  TMRUFiles = class(TComponent)
  private
    FFiles: TStringList;
    FMaxEntries: Byte;
    FMenuItem: TMenuItem;
    FNumbered: Boolean;
    FFullPath: Boolean;
    FHint: String;
    FPrefix: String;                                                            {*we 08-03}
    FMenuItemCount: Word;
    FOnClick: TMRUClickEvent;
    FOnMenuItem: TMRUMenuItemEvent;
    procedure SetMaxEntries(Value: Byte);
    procedure SetMenuItem(Value: TMenuItem);
    procedure SetNumbered(Value: Boolean);
    procedure SetFullPath(Value: Boolean);
    procedure SetHint(const Value: String);
    function IsHintStored: Boolean;
    function IsPrefixStored: Boolean;
    procedure ClearMenuItems;
    procedure FilesChanged(Sender: TObject);
    procedure MenuItemClicked(Sender: TObject);
  protected
    procedure DoClick(Index: Integer); virtual;
    procedure DoMenuItem(Index: Integer; MenuItem: TMenuItem); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Add(const FileName: String);
    procedure Remove(const FileName: String);
    procedure Rename(const OldFileName, NewFileName: String);
    {$IFDEF WIN32}
    procedure LoadFromRegistry(const Key: String);
    procedure SaveToRegistry(const Key: String);
    {$ENDIF}
    procedure LoadFromIni(const IniFileName, Section: String);
    procedure SaveToIni(const IniFileName, Section: String);
    procedure LoadFromFile(const FileName: String);
    procedure SaveToFile(const FileName: String);
    property Files: TStringList read FFiles;
  published
    property MenuItem: TMenuItem read FMenuItem write SetMenuItem;
    property MaxEntries: Byte read FMaxEntries write SetMaxEntries default 9;
    property FullPath: Boolean read FFullPath write SetFullPath default True;
    property Numbered: Boolean read FNumbered write SetNumbered default True;
    property Hint: String read FHint write SetHint stored IsHintStored;
    property Prefix: String read FPrefix write FPrefix stored IsPrefixStored;   {*we 08-03}
    property OnClick: TMRUClickEvent read FOnClick write FOnClick;
    property OnMenuItem: TMRUMenuItemEvent read FOnMenuItem write FOnMenuItem;
  end;

function ReplaceInString(const S, OldPattern, NewPattern: String): String;

procedure Register;

implementation

{$IFDEF WIN32}
  {$R *.D32}
{$ELSE}
  {$R *.D16}
{$ENDIF}

uses
  SysUtils, Forms, IniFiles {$IFDEF WIN32}, Registry {$ENDIF};

const
  SDefHint   = 'Opens %s';
  SDefPrefix = 'File';

{ Helper Functions }

function ReplaceInString(const S, OldPattern, NewPattern: String): String;
var
  I: Integer;
begin
  Result := S;
  for I := Length(Result) - Length(OldPattern) + 1 downto 1 do
    if Copy(Result, I, Length(OldPattern)) = OldPattern then
    begin
      Delete(Result, I, Length(OldPattern));
      Insert(NewPattern, Result, I);
    end;
end;

{ TMRUFiles }

constructor TMRUFiles.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMenuItem := nil;
  FMaxEntries := 9;
  FNumbered := True;
  FFullPath := True;
  FHint := SDefHint;
  FPrefix := SDefPrefix;
  FFiles := TStringList.Create;
  Files.OnChange := FilesChanged;
end;

destructor TMRUFiles.Destroy;
begin
  Files.Free;
  inherited Destroy;
end;

procedure TMRUFiles.Add(const FileName: String);
var
  I: Integer;
begin
  if FileName <> '' then
  begin
    Files.BeginUpdate;
    try
      for I := 0 to Files.Count-1 do
        if CompareText(Files[I], FileName) = 0 then
        begin
          Files.Delete(I);
          Break;
        end;
      while Files.Count >= MaxEntries do
        Files.Delete(MaxEntries-1);
      Files.Insert(0, FileName);
    finally
      Files.EndUpdate;
    end;
  end;
end;

procedure TMRUFiles.Remove(const FileName: String);
var
  I: integer;
begin
  if FileName <> '' then
    for I := 0 to Files.Count-1 do
      if (CompareText(Files[I], FileName) = 0) then
      begin
        Files.Delete(I);
        Break;
      end;
end;

procedure TMRUFiles.Rename(const OldFileName, NewFileName: String);
var
  I: integer;
begin
  if OldFileName <> NewFileName then
    if OldFileName = '' then
      Add(NewFileName)
    else if NewFileName = '' then
      Remove(OldFileName)
    else
      for I := 0 to Files.Count-1 do
        if (CompareText(Files[I], OldFileName) = 0) then
        begin
          Files[I] := NewFileName;
          Break;
        end;
end;

{$IFDEF WIN32}

procedure TMRUFiles.LoadFromRegistry(const Key: String);
var
  I: Integer;
  FileName: String;
  Registry: TRegistry;
begin
  Files.BeginUpdate;
  try
    Files.Clear;
    Registry := TRegistry.Create;
    try
      Registry.RootKey := HKEY_CURRENT_USER;
      if Registry.OpenKey(Key, False) then
      begin
        for I := 1 to MaxEntries do
        begin
          if Registry.ValueExists(Format('%s%d', [Prefix, I])) then             {*we 08-03}
          begin
            FileName := Registry.ReadString(Format('%s%d', [Prefix, I]));       {*we 08-03}
            if (FileName <> '') and (Files.IndexOf(FileName) < 0) then
              Files.Append(FileName);
          end;
        end;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  finally
    Files.EndUpdate;
  end;
end;

procedure TMRUFiles.SaveToRegistry(const Key: String);
var
  I: Integer;
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    Registry.DeleteKey(Key);
    if Registry.OpenKey(Key, True) then
    begin
      for I := 1 to Files.Count do
        Registry.WriteString(Format('%s%d', [Prefix, I]), Files[I-1]);          {*we 08-03}
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;

{$ENDIF}

procedure TMRUFiles.LoadFromIni(const IniFileName, Section: String);
var
  I: Integer;
  FileName: String;
  Ini: TIniFile;
begin
  Files.BeginUpdate;
  try
    Files.Clear;
    Ini := TIniFile.Create(IniFileName);
    try
      for I := 1 to MaxEntries do
      begin
        FileName := Ini.ReadString(Section, Format('%s%d', [Prefix, I]), '');   {*we 08-03}
        if FileName = '' then
          Break
        else if Files.IndexOf(FileName) < 0 then
          Files.Append(FileName);
      end;
    finally
      Ini.Free;
    end;
  finally
    Files.EndUpdate;
  end;
end;

procedure TMRUFiles.SaveToIni(const IniFileName, Section: String);
var
  I: Integer;
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(IniFileName);
  try
    Ini.EraseSection(Section);
    for I := 1 to Files.Count do
      Ini.WriteString(Section, Format('%s%d', [Prefix, I]), Files[I-1]);        {*we 08-03}
  finally
    Ini.Free;
  end;
end;

procedure TMRUFiles.LoadFromFile(const FileName: String);
begin
  Files.BeginUpdate;
  try
    Files.Clear;
    Files.LoadFromFile(FileName);
  finally
    Files.EndUpdate;
  end;
end;

procedure TMRUFiles.SaveToFile(const FileName: String);
begin
  Files.SaveToFile(FileName);
end;

procedure TMRUFiles.SetMaxEntries(Value: Byte);
begin
  if MaxEntries <> Value then
  begin
    FMaxEntries := Value;
    Files.BeginUpdate;
    try
      while Files.Count > MaxEntries do
        Files.Delete(MaxEntries-1);
    finally
      Files.EndUpdate;
    end;
  end;
end;

procedure TMRUFiles.SetNumbered(Value: Boolean);
begin
  if Numbered <> Value then
  begin
    FNumbered := Value;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.SetFullPath(Value: Boolean);
begin
  if FullPath <> Value then
  begin
    FFullPath := Value;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.SetHint(const Value: String);
begin
  if Hint <> Value then
  begin
    FHint := Value;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.SetMenuItem(Value: TMenuItem);
begin
  if MenuItem <> Value then
  begin
    ClearMenuItems;
    FMenuItem := Value;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.ClearMenuItems;
var
  I: Integer;
  Index: Integer;
begin
  if MenuItem <> nil then
  begin
    Index := MenuItem.Parent.IndexOf(MenuItem) + 1;
    for I := 1 to FMenuItemCount do
      MenuItem.Parent.Delete(Index);
    FMenuItemCount := 0;
  end;
end;

function TMRUFiles.IsPrefixStored: Boolean;
begin
  Result := (Prefix <> SDefPrefix);
end;

function TMRUFiles.IsHintStored: Boolean;
begin
  Result := (Hint <> SDefHint);
end;

procedure TMRUFiles.FilesChanged(Sender: TObject);
var
  I: Integer;
  Index: Integer;
  NewMenuItem: TMenuItem;
  FileName: String;
begin
  if MenuItem <> nil then
  begin
    ClearMenuItems;
    Index := MenuItem.Parent.IndexOf(MenuItem) + 1;
    for I := Files.Count-1 downto 0 do
    begin
      if FullPath then
        FileName := ReplaceInString(Files[I], '&', '&&')                        {*we 08-03}
      else
        FileName := ReplaceInString(ExtractFileName(Files[I]), '&', '&&');      {*we 08-03}
      NewMenuItem := TMenuItem.Create(Self);
      if not Numbered then
        NewMenuItem.Caption := Format('%s', [FileName])
      else if I < 10 then
        NewMenuItem.Caption := Format('&%d %s', [I+1, FileName])
      else
        NewMenuItem.Caption := Format('  %s', [FileName]);
      NewMenuItem.Hint := Format(Hint, [Files[I]]);
      DoMenuItem(I, NewMenuItem);
      NewMenuItem.Tag := I;
      NewMenuItem.OnClick := MenuItemClicked;
      MenuItem.Parent.Insert(Index, NewMenuItem);
      Inc(FMenuItemCount);
    end;
    if Files.Count > 0 then
    begin
      NewMenuItem := TMenuItem.Create(Self);
      NewMenuItem.Caption := '-';
      MenuItem.Parent.Insert(Index, NewMenuItem);
      Inc(FMenuItemCount);
    end;
  end;
end;

procedure TMRUFiles.DoClick(Index: Integer);
begin
  if Assigned(FOnClick) then
    FOnClick(Self, Files[Index]);
end;

procedure TMRUFiles.DoMenuItem(Index: Integer; MenuItem: TMenuItem);
begin
  if Assigned(FOnMenuItem) then
    FOnMenuItem(Self, Index, MenuItem);
end;

procedure TMRUFiles.MenuItemClicked(Sender: TObject);
begin
  if Sender is TMenuItem then
    DoClick(TMenuItem(Sender).Tag);
end;

procedure Register;
begin
  RegisterComponents('Delphi Area', [TMRUFiles]);
end;

end.
